
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include "textures.h"


// load a 256x256 RGB .RAW file as a texture
GLuint LoadTextureRAW( const char * filename, int wrap )
{
  GLuint texture;
  int width, height;
  char * data;
  FILE * file;
  
  // open texture data
  file = fopen( filename, "rb" );
  if ( file == NULL ) return 0;
  
  // allocate buffer
  width = 256;
  height = 256;
  data = (char *)malloc( width * height * 3 );
  
  // read texture data
  fread( data, width * height * 3, 1, file );
  fclose( file );
  
  // allocate a texture name
  glGenTextures( 1, &texture );
  
  // select our current texture
  glBindTexture( GL_TEXTURE_2D, texture );
  
  // select modulate to mix texture with color for shading
  glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
  
  // when texture area is small, bilinear filter the closest MIP map
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
		   GL_LINEAR_MIPMAP_NEAREST );
  // when texture area is large, bilinear filter the first MIP map
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  
  // if wrap is true, the texture wraps over at the edges (repeat)
  //       ... false, the texture ends at the edges (clamp)
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
		   wrap ? GL_REPEAT : GL_CLAMP );
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
		   wrap ? GL_REPEAT : GL_CLAMP );
  
  // build our texture MIP maps
  gluBuild2DMipmaps( GL_TEXTURE_2D, 3, width,
		     height, GL_RGB, GL_UNSIGNED_BYTE, data );
  
  // free buffer
  free( data );
  
  return texture;
  
}


void LoadTGATexture(Texture *texture, char *filename)
{    
  // Targa data type -- uncompressed, RGBA image; element
  // 3 is always 2.
  //GLubyte    targaMagic[12] = { 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
  GLubyte    targaMagic[12] = { 42, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
  
  GLubyte    fileMagic[12];

  // Header data.
  GLubyte    header[6];
  GLuint     bytesPerPixel;
  
  GLuint     imageSize;
  GLint      sizeRead;
  GLuint     tmp;
  GLint      i;
  
  FILE *file = fopen(filename, "rb");
  
  // Start reading in the header, make sure we
  // have the right Targa data type.  
  
  if(file == NULL)
    {
      //      printf("1");
      fclose(file);
      return;
    }

  if ( fread(fileMagic, 1, sizeof(fileMagic), file)      != sizeof(fileMagic))
    {
      //      printf("2");
      fclose(file);
      return;
    }

  if(memcmp(targaMagic, fileMagic, sizeof(targaMagic)) != 0)
    {
      //      printf("3: wanted: ");
      //for( i = 0; i < 12; i++)
	//	printf("%d ", targaMagic[i]);
      //printf("\tGot: ");
      //for( i = 0; i < 12; i++)
      //printf("%d ", fileMagic[i]);

      fclose(file);
      return;
    }

  if(fread(header, 1, sizeof(header), file)            != sizeof(header))
    {
      //printf("4");
      fclose(file);
      return;
    }

  //  printf("WE're okay!\n");

  // Determine width and height from header data.
  
  texture->width  = header[1] * 256 + header[0];
  texture->height = header[3] * 256 + header[2];
  
  // Determine color bit depth of the image (24 or 32).
  
  if(texture->width <= 0 || texture->height <= 0 || (header[4] != 24 && header[4] != 32))
    {
      fclose(file);
      return;
    }
  
  texture->bpp  = header[4];
  bytesPerPixel = texture->bpp / 8;
  imageSize     = texture->width * texture->height * bytesPerPixel;
  
  texture->imageData = (GLubyte *)malloc(imageSize);
  
  if(texture->imageData == NULL ||
     fread(texture->imageData, 1, imageSize, file) != imageSize)
    {
      if(texture->imageData != NULL)
	free(texture->imageData);
      
      fclose(file);
      return;
    } 
  
  for(i = 0; i < imageSize; i += bytesPerPixel)
    {
      tmp = texture->imageData[i];
      texture->imageData[i] = texture->imageData[i + 2];
      texture->imageData[i + 2] = tmp;
    }
  
  fclose (file);



  // allocate a texture name
  glGenTextures( 1, &(texture->texID) );
  
  // select our current texture
  glBindTexture( GL_TEXTURE_2D, (texture->texID) );
  
  // select modulate to mix texture with color for shading
  glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
  
  // when texture area is small, bilinear filter the closest MIP map
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
		   GL_LINEAR_MIPMAP_NEAREST );
  // when texture area is large, bilinear filter the first MIP map
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  
  // if wrap is true, the texture wraps over at the edges (repeat)
  //       ... false, the texture ends at the edges (clamp)
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
		   GL_REPEAT);
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
		   GL_REPEAT);
  
  // build our texture MIP maps
  gluBuild2DMipmaps( GL_TEXTURE_2D, 3, texture->width,
		     texture->height, GL_RGB, GL_UNSIGNED_BYTE, texture->imageData );


}
